#property copyright "ArteInVolo"
#property link      "https://vk.com/tradelearning"
#property version   "1.0"

#property indicator_chart_window
#property indicator_buffers 2
#property indicator_plots   2

#property indicator_type1   DRAW_ARROW
#property indicator_color1  Blue
#property indicator_style1  STYLE_SOLID
#property indicator_width1  1

#property indicator_type2   DRAW_ARROW
#property indicator_color2  Red
#property indicator_style2  STYLE_SOLID
#property indicator_width2  1

input int InpLeftSide = 10;
input int InpRightSide = 5;
input int InpNumberOfBars = 200;
input int InpLevelsDistance = 0;
input bool InpShowFractals = false;
input color InpLevelColor = DarkGray;
input ENUM_TIMEFRAMES InpPeriod = PERIOD_CURRENT;

double FractalUp[];
double FractalDown[];

int LeftSide, RightSide, MaxSide;
string LevelName = "FractalLevel";

int OnInit() 
{
    LeftSide = MathMax(InpLeftSide, 2);
    RightSide = MathMax(InpRightSide, 2);
    MaxSide = MathMax(LeftSide, RightSide);
    
    SetIndexBuffer(0, FractalUp, INDICATOR_DATA);
    SetIndexBuffer(1, FractalDown, INDICATOR_DATA);
    
    PlotIndexSetInteger(0, PLOT_ARROW, 217);
    PlotIndexSetInteger(1, PLOT_ARROW, 218);
    
    PlotIndexSetInteger(0, PLOT_ARROW_SHIFT, -10);  
    PlotIndexSetInteger(1, PLOT_ARROW_SHIFT, 10);
    
    IndicatorSetInteger(INDICATOR_DIGITS, _Digits);
    IndicatorSetString(INDICATOR_SHORTNAME, "FractalLevels");
    
    return(0);
}

void OnDeinit(const int reason) 
{
    HideLines();
}

int OnCalculate(
    const int rates,
    const int counted,
    const datetime &Time[],
    const double &Open[],
    const double &High[],
    const double &Low[],
    const double &Close[],
    const long &TickVolume[],
    const long &Volume[],
    const int &Spread[])
{
    int bars = MathMin(InpNumberOfBars, rates);

    ArraySetAsSeries(Time, true);
    ArraySetAsSeries(Open, true);
    ArraySetAsSeries(High, true);
    ArraySetAsSeries(Low, true);
    ArraySetAsSeries(Close, true);
    ArraySetAsSeries(TickVolume, true);
    ArraySetAsSeries(Volume, true);
    ArraySetAsSeries(Spread, true);
    
    ArraySetAsSeries(FractalUp, true);
    ArraySetAsSeries(FractalDown, true);
    
    ArrayInitialize(FractalUp, 0.0);
    ArrayInitialize(FractalDown, 0.0);

    double periodHigh[], periodLow[];
    
    CopyHigh(_Symbol, InpPeriod, 0, bars, periodHigh);
    CopyLow(_Symbol, InpPeriod, 0, bars, periodLow);
    
    // Clean up levels

    HideLines();
    
    // Look up for new levels
    
    DrawLines(bars, periodHigh, periodLow);
    
    return(rates);
}

int ShowLine(const string name, const double price)
{
    if (ObjectFind(0, name) < 0)
    {
        ObjectCreate(0, name, OBJ_HLINE, 0, 0, price);
    }
    
    ObjectSetInteger(0, name, OBJPROP_COLOR, InpLevelColor);
    ObjectSetInteger(0, name, OBJPROP_WIDTH, 1);
    ObjectSetInteger(0, name, OBJPROP_STYLE, STYLE_DOT);
    ObjectSetInteger(0, name, OBJPROP_BACK, true);

    return(0);
}

int DrawLines(int bars, const double &High[], const double &Low[])
{
    double levels[];

    for (int bar = 0; bar < bars; bar++) 
    {
        // Look for Fractals on Tops
    
        if (isFractal(bar, true, High)) 
        {
            int size = ArraySize(levels);

            ArrayResize(levels, size + 1);

            levels[size] = High[bar];
            
            if (InpShowFractals)
            {
                FractalUp[bar] = levels[size];
            }
        } 
        
        // Look for Fractals on Bottoms
        
        if (isFractal(bar, false, Low)) 
        {
            int size = ArraySize(levels);
            
            ArrayResize(levels, size + 1);
            
            levels[size] = Low[bar];
            
            if (InpShowFractals)
            {
                FractalDown[bar] = levels[size];
            }
        } 
    }

    int levelsSize = ArraySize(levels);
    double factor = InpLevelsDistance > 0 ? InpLevelsDistance * Point() : Distance(levels), min = 9000000000;

    ArraySort(levels);
    ArraySetAsSeries(levels, true);
    
    for (int i = 0; i < levelsSize; i++) 
    {
        if (min - factor > levels[i])
        {
            min = levels[i];
            ShowLine(StringFormat("FractalLevel-%i-%f", i, levels[i]), levels[i]);
        }
    }
    
    return(0);
}

int HideLines()
{
    ObjectsDeleteAll(0, 0, OBJ_HLINE);
    
    return(0);
}

bool isFractal(int bar, bool isUp, const double &list[])
{
    int count = ArraySize(list);

    for (int i = 1; i < MaxSide && bar + i < count && bar - i > -1; i++) 
    {
        // When it is Up Fractal look for closest bars to be sure that they are lower than current
    
        bool condition = isUp ? 
            i < LeftSide && list[bar + i] >= list[bar] || i < RightSide && list[bar - i] >= list[bar] : 
            i < LeftSide && list[bar + i] <= list[bar] || i < RightSide && list[bar - i] <= list[bar];
    
        if (condition)
        {
            return(false);
        }
    }
    
    return(bar > 0 ? true : false);
}

double Distance(const double &levels[])
{
    double max = 0, min = 9000000000, size = ArraySize(levels), up = 0, down = 0;

    for (int i = 0; i < size; i++) 
    {
        if (max < levels[i])
        {
            up++;
            max = levels[i];
        }
        
        if (min > levels[i])
        {
            down++;
            min = levels[i];
        }
    }

    return (max - min) / (up + down);
}